@@ -27,6 +27,8 @@ from git import (Repo, Commit, InvalidGitRepositoryError,  # type: ignore
                  GitCommandError, BadName, RemoteReference) # type: ignore
 from atomicwrites import atomic_write
 
+from mathlibtools.file_status import FileStatus, PortStatus
+
 if TYPE_CHECKING:
     from mathlibtools.import_graph import ImportGraph
 
@@ -132,6 +134,13 @@ def escape_identifier(s : str) -> str:
         return s
     return "«" + s + "»"
 
+def blocks(files, size=65536):
+    """Help function to count file lines"""
+    while True:
+        b = files.read(size)
+        if not b: break
+        yield b
+
 class OleanCache:
     """ A reference to a cache of oleans for a single commit.
 
@@ -290,13 +299,13 @@ class CacheLocator:
             if len(caches) > 1:
                 archive_items = ''.join([f'\n * {short_sha(c.rev)}' for c in caches])
                 commit_args = ''.join([f' {short_sha(c.rev)}^!' for c in caches])
-                log.warn(
+                log.warning(
                     f"No cache was available for {short_sha(rev)}.\n"
                     f"There are multiple viable caches from parent commits:{archive_items}\n"
                     f"To see the commits in question, run:\n"
                     f"  git log --graph {short_sha(rev)}{commit_args}")
             else:
-                log.warn(
+                log.warning(
                     f"No cache was available for {short_sha(rev)}. "
                     f"A cache was found for the ancestor {short_sha(cache.rev)}.\n"
                     f"To see the intermediate commits, run:\n"
@@ -423,12 +432,13 @@ class LeanProject:
                   force_download: bool = False,
                   upgrade_lean: bool = True) -> 'LeanProject':
         """Builds a LeanProject from a Path object"""
+        repo: Optional[Repo] = None
+        is_dirty = False
+        rev = ''
         try:
             repo = Repo(path, search_parent_directories=True)
         except InvalidGitRepositoryError:
-            repo = None
-            is_dirty = False
-            rev = ''
+            pass
         if repo:
             if repo.bare:
                 raise InvalidLeanProject('Git repository is not initialized')
@@ -560,17 +570,18 @@ class LeanProject:
         mathlib)"""
         if self.is_mathlib:
             # user should have run `get-cache` not `get-mathlib-cache
-            log.warn("`get-mathlib-cache` is for projects which depend on "
-                     "mathlib, not for mathlib itself. "
-                     "Running `get-cache` instead.")
+            log.warning(
+                "`get-mathlib-cache` is for projects which depend on "
+                "mathlib, not for mathlib itself. "
+                "Running `get-cache` instead.")
             return self.get_cache()
 
         repo = self.mathlib_repo
         try:
             commit = repo.rev_parse(self.mathlib_rev)
-        except BadName:
+        except (BadName, ValueError):
             # presumably the mathlib folder is outdated
-            log.info("Can't find the required mathlib revision, will try to update"
+            log.info("Can't find the required mathlib revision, will try to update "
                      "mathlib git repository")
             self.run_echo(['leanpkg', 'configure'])
             commit = repo.rev_parse(self.mathlib_rev)
@@ -580,7 +591,7 @@ class LeanProject:
         if not (self.directory/'leanpkg.path').exists():
             self.run(['leanpkg', 'configure'])
 
-        cache_locator = CacheLocator(self.name, repo, self.cache_url, DOT_MATHLIB,
+        cache_locator = CacheLocator('mathlib', repo, self.cache_url, DOT_MATHLIB,
                                      force_download=self.force_download)
 
         # We want an exact match here; if we can't find one, then the user should
@@ -670,6 +681,7 @@ class LeanProject:
                 log.error('Invalid git branch')
                 shutil.rmtree(target)
                 raise err
+        assert repo.working_dir is not None
         proj = cls.from_path(Path(repo.working_dir), cache_url, force_download)
         proj.run_echo(['leanpkg', 'configure'])
         if 'mathlib' in proj.deps or proj.is_mathlib:
@@ -799,7 +811,7 @@ class LeanProject:
             raise LeanProjectError('This project has no git repository.')
         hook_dir = Path(self.repo.git_dir)/'hooks'
         src = Path(__file__).parent
-        log.info('This script will copy post-commit and post-checkout scripts to ', hook_dir)
+        log.info('This script will copy post-commit and post-checkout scripts to %s', hook_dir)
         rep = input("Do you want to proceed (y/n)? ")
         if rep in ['y', 'Y']:
             shutil.copy(str(src/'post-commit'), str(hook_dir))
@@ -845,8 +857,6 @@ class LeanProject:
                     continue
                 imp_label = str(imp_rel.with_suffix('')).replace(os.sep, '.')
                 G.add_edge(imp_label, label)
-        for node in G:
-            G.nodes[node]['label'] = node
         self._import_graph = G
         return G
 
@@ -951,7 +961,7 @@ class LeanProject:
             raise LeanProjectError('This operation is for mathlib only.')
         if not self.repo:
             raise LeanProjectError('This project has no git repository.')
-        if branch_name in self.repo.branches:
+        if branch_name in self.repo.branches: # type: ignore
             raise LeanProjectError(f'The branch {branch_name} already exists, '
                                     'please choose another name.')
         log.info('Checking out master...')
@@ -997,6 +1007,7 @@ class LeanProject:
         if self.is_dirty:
             raise LeanDirtyRepo('Cannot pull because repository is dirty')
         old_mathlib = self.mathlib_rev
+        assert self.repo
         log.info(f"Pulling from {remote}")
         self.repo.remote(remote).pull(self.repo.active_branch)
         self.read_config()
@@ -1008,3 +1019,90 @@ class LeanProject:
                 self.run_echo(['leanpkg', 'configure'])
             self.get_mathlib_olean()
 
+    def port_status(self, url: Optional[str] = None, mathlib4: Optional[Path] = None) -> None:
+        """
+        Color nodes on the graph based on port status. Done in place to the graph nodes.
+
+        Args:
+            url: md or yaml file with "file: label" content, by default, from the wiki
+        """
+
+        def snake_to_camel(filename: str) -> str:
+            return re.sub('(?!^)([A-Z]+)', r'_\1', filename).lower()
+
+        existing_files = set()
+        if mathlib4:
+            mathlib4 = mathlib4 / "Mathlib"
+            existing_files = {str.join(".",
+                map(snake_to_camel, leanfile.relative_to(mathlib4).with_suffix("").parts)).lower()
+                for leanfile in mathlib4.rglob("*") if leanfile.suffix == ".lean"}
+
+        port_status = PortStatus.deserialize_old()
+
+        for node_name, node in self.import_graph.nodes(data=True):
+            node["status"] = port_status.file_statuses.get(node_name, FileStatus())
+            node_path = self.src_directory.joinpath(*node_name.split(".")).with_suffix(".lean")
+            with open(node_path, "r",encoding="utf-8",errors='ignore') as f:
+                node["nb_lines"] = sum(bl.count("\n") for bl in blocks(f))
+        # somehow missing from yaml
+        # for node_name, node in self.import_graph.nodes(data=True):
+        #     if node_name not in port_labels:
+        #         node["status"] = FileStatus.missing()
+        finished_nodes = {node for node, attrs in self.import_graph.nodes(data=True)
+                          if attrs.get("status").ported}
+        # tag nodes that have finished parents, depth of 1
+        for node in finished_nodes:
+            # does not get root nodes because they are not at end of an out_edge
+            for _, target in self.import_graph.out_edges(node):
+                # we don't need to redo a finished node
+                if target in finished_nodes:
+                    continue
+                parents = {parent for parent, _ in self.import_graph.in_edges(target)}
+                if parents.issubset(finished_nodes):
+                    target_node = self.import_graph.nodes[target]
+                    target_node["status"].comments = "ready"
+        # now to get root nodes
+        for target, degree in self.import_graph.in_degree():
+            target_node = self.import_graph.nodes[target]
+            if degree > 0 or target_node["status"].comments:
+                continue
+            target_node["status"].comments = "ready"
+        for node_name, node in self.import_graph.nodes(data=True):
+            if not node.get("status"):
+                continue
+            node["style"] = "filled"
+            file_status: FileStatus = node["status"]
+            node["fillcolor"] = node.get("fillcolor", self.status_color(file_status))
+            if file_status.pr_link:
+                node["href"] = '"' + file_status.pr_link + '"'
+            if node_name in existing_files and not file_status.ported:
+                node["color"] = "red"
+
+    @staticmethod
+    def status_color(status: FileStatus) -> Optional[str]:
+        """
+        How each status should be colored, should be applied to the node attrs at "fillcolor".
+        """
+        if status.ported:
+            return "green"
+        if status.mathlib4_pr:
+            return "lightskyblue"
+        if status.comments and "ready" in status.comments:
+            return "turquoise1"
+        return None
+
+    def modules_used(self, module: str) -> List[str]:
+        """
+        A wrapper around the 'scripts/modules_used.lean' script in the mathlib repository.
+        """
+        self.make_all()
+        return self.run(["lean", "--run", "scripts/modules_used.lean", module]).split()
+
+    def show_unused(self, to: str) -> None:
+        """Highlight any files which are not needed by the file 'to'."""
+        used = self.modules_used(to)
+        for label, node in self.import_graph.nodes(data=True):
+            if not (label.startswith("tactic.") or label.startswith("meta.") or label in used):
+                node["style"] = "filled"
+                if node.get("fillcolor") is None:
+                    node["fillcolor"] = "#EEEEEE"
